@@ -1,12 +1,20 @@
+"""Root DipDup package.
+
+Contains software versions and other metadata. Also applies some CLI environment hacks on import.
+"""
 import logging
 import sys
 import warnings
 from pathlib import Path
 
 # NOTE: Updated by bumpversion
-__version__ = '6.2.0'
+__version__ = '6.3.0'
 __spec_version__ = '1.2'
 
+version = tuple(__version__.split('.'))
+major_version = version[0]
+minor_version = f'{version[0]}.{version[1]}'
+
 spec_version_mapping = {
     '0.1': '<=0.4.3',
     '1.0': '>=1.0.0, <=1.1.2',
