@@ -9,6 +9,7 @@ from contextlib import AsyncExitStack
 from contextlib import suppress
 from functools import partial
 from functools import wraps
+from os import environ as env
 from pathlib import Path
 from typing import Any
 from typing import Dict
@@ -237,7 +238,9 @@ def _init_sentry(config) -> None:
 
 async def _check_version() -> None:
     if 'rc' in __version__:
-        _logger.warning('You are running a pre-release version of DipDup. Please, report any issues to the GitHub repository.')
+        _logger.warning(
+            'You are running a pre-release version of DipDup. Please, report any issues to the GitHub repository.'
+        )
         _logger.info('Set `skip_version_check` flag in config to hide this message.')
         return
 
@@ -246,7 +249,7 @@ async def _check_version() -> None:
     async with AsyncExitStack() as stack:
         stack.enter_context(suppress(Exception))
         session = await stack.enter_async_context(aiohttp.ClientSession())
-        response = await session.get('https://api.github.com/repos/dipdup-net/dipdup/releases/latest')
+        response = await session.get('https://api.github.com/repos/dipdup-io/dipdup/releases/latest')
         response_json = await response.json()
         latest_version = response_json['tag_name']
 
@@ -282,8 +285,12 @@ async def cli(ctx, config: List[str], env_file: List[str]):
 
     Documentation: https://docs.dipdup.io
 
-    Issues: https://github.com/dipdup-net/dipdup/issues
+    Issues: https://github.com/dipdup-io/dipdup/issues
     """
+    # TODO: Remove in 7.0
+    if env.get('DIPDUP_PYTEZOS'):
+        _logger.warning('PyTezos extra and corresponding Docker image is deprecated!')
+
     # NOTE: Workaround for help pages
     args = sys.argv[1:]
     if '--help' in args or args in ([], ['config'], ['hasura'], ['schema']):
@@ -583,7 +590,10 @@ async def schema_wipe(ctx, immune: bool, force: bool) -> None:
     if not force:
         try:
             assert sys.__stdin__.isatty()
-            click.confirm(f'You\'re about to wipe schema `{url}`. All indexed data will be irreversibly lost, are you sure?', abort=True)
+            click.confirm(
+                f"You're about to wipe schema `{url}`. All indexed data will be irreversibly lost, are you sure?",
+                abort=True,
+            )
         except AssertionError:
             click.echo('Not in a TTY, skipping confirmation')
         except click.Abort:
@@ -635,7 +645,10 @@ async def schema_init(ctx) -> None:
 
         # NOTE: It's not necessary a reindex, but it's safe to execute built-in scripts to (re)create views.
         conn = get_connection()
-        await generate_schema(conn, config.database.schema_name)
+        await generate_schema(
+            conn,
+            config.database.schema_name,
+        )
 
     _logger.info('Schema initialized')
 
