@@ -251,17 +251,25 @@ class IndexDispatcher:
         if not operation_subgroups:
             return
 
-        operation_indexes = (i for i in self._indexes.values() if isinstance(i, OperationIndex) and i.datasource == datasource)
+        operation_indexes = (
+            i for i in self._indexes.values() if isinstance(i, OperationIndex) and i.datasource == datasource
+        )
         for index in operation_indexes:
             index.push_operations(operation_subgroups)
 
-    async def _on_token_transfers(self, datasource: IndexDatasource, token_transfers: Tuple[TokenTransferData, ...]) -> None:
-        token_transfer_indexes = (i for i in self._indexes.values() if isinstance(i, TokenTransferIndex) and i.datasource == datasource)
+    async def _on_token_transfers(
+        self, datasource: IndexDatasource, token_transfers: Tuple[TokenTransferData, ...]
+    ) -> None:
+        token_transfer_indexes = (
+            i for i in self._indexes.values() if isinstance(i, TokenTransferIndex) and i.datasource == datasource
+        )
         for index in token_transfer_indexes:
             index.push_token_transfers(token_transfers)
 
     async def _on_big_maps(self, datasource: IndexDatasource, big_maps: Tuple[BigMapData, ...]) -> None:
-        big_map_indexes = (i for i in self._indexes.values() if isinstance(i, BigMapIndex) and i.datasource == datasource)
+        big_map_indexes = (
+            i for i in self._indexes.values() if isinstance(i, BigMapIndex) and i.datasource == datasource
+        )
         for index in big_map_indexes:
             index.push_big_maps(big_maps)
 
@@ -270,7 +278,9 @@ class IndexDispatcher:
         for index in event_indexes:
             index.push_events(events)
 
-    async def _on_rollback(self, datasource: IndexDatasource, type_: MessageType, from_level: int, to_level: int) -> None:
+    async def _on_rollback(
+        self, datasource: IndexDatasource, type_: MessageType, from_level: int, to_level: int
+    ) -> None:
         """Call `on_index_rollback` hook for each index that is affected by rollback"""
         if from_level <= to_level:
             raise RuntimeError(f'Attempt to rollback forward: {from_level} -> {to_level}')
@@ -296,22 +306,12 @@ class IndexDispatcher:
                 self._logger.debug('%s: level is too low, skipping', index_name)
 
             else:
-                self._logger.debug('%s: unprocessed', index_name)
+                self._logger.debug('%s: affected', index_name)
                 affected_indexes.add(index_name)
 
-        self._logger.info(
-            '%s/%s indexes affected',
-            len(affected_indexes),
-            len(self._indexes),
-        )
-
-        if not affected_indexes:
-            self._logger.info('`%s` rollback complete', channel)
-            return
-
         hook_name = 'on_index_rollback'
         for index_name in affected_indexes:
-            self._logger.warning('`%s`: can\'t process, firing `%s` hook', index_name, hook_name)
+            self._logger.warning('`%s` index is affected by rollback; firing `%s` hook', index_name, hook_name)
             await self._ctx.fire_hook(
                 hook_name,
                 index=self._indexes[index_name],
@@ -319,6 +319,8 @@ class IndexDispatcher:
                 to_level=to_level,
             )
 
+        self._logger.info('`%s` rollback complete', channel)
+
 
 class DipDup:
     """Main indexer class.
@@ -395,7 +397,9 @@ class DipDup:
             spawn_index_tasks = (create_task(self._ctx._spawn_index(name)) for name in self._config.indexes)
             await gather(*spawn_index_tasks)
 
-            await self._set_up_index_dispatcher(tasks, spawn_datasources_event, start_scheduler_event, advanced_config.early_realtime)
+            await self._set_up_index_dispatcher(
+                tasks, spawn_datasources_event, start_scheduler_event, advanced_config.early_realtime
+            )
 
             await gather(*tasks)
 
@@ -430,8 +434,11 @@ class DipDup:
             except KeyError:
                 await self._ctx.reindex(ReindexingReason.schema_modified)
 
-        # NOTE: Call even if Schema is present; there may be new tables
-        await generate_schema(conn, schema_name)
+        # NOTE: Call even if Schema is present to create new tables
+        await generate_schema(
+            conn,
+            schema_name,
+        )
         schema_hash = get_schema_hash(conn)
 
         if self._schema is None:
