@@ -82,7 +82,7 @@ class DipDupError(Exception):
         return """
             An unexpected error has occurred!
 
-            Please file a bug report at https://github.com/dipdup-net/dipdup/issues
+            Please file a bug report at https://github.com/dipdup-io/dipdup/issues
         """
 
     def help(self) -> str:
@@ -103,7 +103,7 @@ class DatasourceError(DipDupError):
         return f"""
             `{self.datasource}` datasource returned an error: {self.msg}
 
-            Please file a bug report at https://github.com/dipdup-net/dipdup/issues
+            Please file a bug report at https://github.com/dipdup-io/dipdup/issues
         """
 
 
@@ -139,9 +139,10 @@ class ConfigurationError(DipDupError):
 
 
 @dataclass(repr=False)
-class InvalidModelsError(ConfigurationError):
+class InvalidModelsError(DipDupError):
     """Can't initialize database, `models.py` module is invalid"""
 
+    msg: str
     model: Type[Model]
     field: Optional[str] = None
 
@@ -159,6 +160,27 @@ class InvalidModelsError(ConfigurationError):
         """
 
 
+@dataclass(repr=False)
+class DatabaseEngineError(DipDupError):
+    """Some of the features are not supported with the current database engine"""
+
+    msg: str
+    kind: str
+    required: str
+
+    def _help(self) -> str:
+        return f"""
+            {self.msg}
+
+              database: `{self.kind}`
+              required: `{self.required}`
+
+            See https://docs.dipdup.io/deployment/database-engines
+            See https://docs.dipdup.io/advanced/sql
+            See https://docs.dipdup.io/config/database
+        """
+
+
 @dataclass(repr=False)
 class MigrationRequiredError(DipDupError):
     """Project and DipDup spec versions don't match"""
