@@ -3,7 +3,6 @@ import logging
 import pkgutil
 import types
 from collections import defaultdict
-from contextlib import suppress
 from decimal import Decimal
 from functools import reduce
 from logging import Logger
@@ -96,12 +95,14 @@ class FormattedLogger(Logger):
 
 def iter_files(path: Path, ext: Optional[str] = None) -> Iterator[TextIO]:
     """Iterate over files in a directory. Or a single file. Sort alphabetically, filter by extension, skip empty files."""
+    if not path.exists() and ext:
+        path = Path(f'{path}{ext}')
     if not path.exists():
         return
     elif path.is_file():
         paths = [path]
     elif path.is_dir():
-        paths = list(path.glob('**/*'))
+        paths = sorted(path.glob('**/*'))
     else:
         raise RuntimeError(f'Path `{path}` exists but is neither a file nor a directory')
 
@@ -124,14 +125,6 @@ def import_from(module: str, obj: str) -> Any:
         raise ProjectImportError(module, obj) from e
 
 
-def is_importable(module: str, obj: str) -> bool:
-    """Check if object can be imported from module"""
-    with suppress(ProjectImportError):
-        import_from(module, obj)
-        return True
-    return False
-
-
 def exclude_none(config_json: Any) -> Any:
     if isinstance(config_json, (list, tuple)):
         return [exclude_none(i) for i in config_json if i is not None]
